// Registry.cpp
#define   UNICODE
#define   _UNICODE
//#define MYDEBUG
#include  <objbase.h>
#include  <tchar.h>
#include  <cstdio>
#include  "Registry.h"
const int CLSID_STRING_BUFFER_LENGTH = 39; //A CLSID Converted Needs 38 chars, i.e., {30000000-0000-0000-0000-000000000000}. The 39 number is for the terminating NULL.
#if defined MYDEBUG
extern FILE* fp;
#endif


BOOL SetKeyAndValue(const TCHAR* szKey, const TCHAR* szSubkey, const TCHAR* szValue)
{
 TCHAR szKeyBuf[1024];
 long lResult;
 HKEY hKey;

 #if defined MYDEBUG
 _ftprintf(fp,_T("    Entering SetKeyAndValue()\n"));
 _ftprintf(fp,_T("      szKey    = %s\n"),szKey);
 _ftprintf(fp,_T("      szSubkey = %s\n"),szSubkey);
 _ftprintf(fp,_T("      szValue  = %s\n"),szValue);
 #endif
 _tcscpy(szKeyBuf,szKey);
 if(szSubkey!=NULL)          // Add subkey name to buffer.
 {
     _tcscat(szKeyBuf, _T("\\"));
     _tcscat(szKeyBuf, szSubkey);
     #if defined MYDEBUG
     _ftprintf(fp,_T("      szKeyBuf = %s\n"),szKeyBuf);
     #endif
 }
 //Create and open key and subkey.
 lResult=RegCreateKeyEx(HKEY_CLASSES_ROOT,szKeyBuf,0,NULL,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&hKey,NULL);
 if(lResult!=ERROR_SUCCESS)
    return FALSE ;
 if(szValue!=NULL)         //Set the Value.
    RegSetValueEx(hKey,NULL,0,REG_SZ,(BYTE*)szValue,(_tcslen(szValue)*sizeof(TCHAR))+sizeof(TCHAR));
 RegCloseKey(hKey);
 #if defined MYDEBUG
 _ftprintf(fp,_T("    Leaving SetKeyAndValue()\n"));
 #endif

 return TRUE;
}


void CLSIDToTChar(const CLSID& clsid, TCHAR* szCLSID, int iStrlen)    // Convert a CLSID to a TCHAR string.
{
 LPOLESTR wszCLSID=NULL;
 size_t pRet=0;
 HRESULT hr;

 #if defined MYDEBUG
 _ftprintf(fp,_T("    Entering CLSIDToTChar()\n"));
 #endif
 hr=StringFromCLSID(clsid,&wszCLSID);      // Get CLSID
 #if defined MYDEBUG
 _ftprintf(fp,_T("      iStrLen  = %d\n"),iStrlen);
 fwprintf(fp,   L"      wszCLSID = %s\n",wszCLSID);
 #endif
 if(SUCCEEDED(hr))
 {
    #if defined UNICODE
        _tcscpy_s(szCLSID,iStrlen,wszCLSID);
        #if defined MYDEBUG
        _ftprintf(fp,_T("      szCLSID  = %s\n"),szCLSID);
        #endif
    #else
        wcstombs_s(&pRet,szCLSID,iStrlen,wszCLSID,iStrlen); // Covert from wide characters to non-wide.
        #if defined MYDEBUG
        _ftprintf(fp,  _T("      szCLSID  = %s\n"),szCLSID);
        _ftprintf(fp,  _T("      pRet     = %u\n"),pRet);
        #endif
    #endif
    CoTaskMemFree(wszCLSID);               // Free memory.
 }
 #if defined MYDEBUG
 _ftprintf(fp,_T("    Leaving CLSIDToTChar()\n"));
 #endif
}


LONG RecursiveDeleteKey(HKEY hKeyParent, const TCHAR* lpszKeyChild)       // Key to delete
{
 TCHAR szBuffer[256];
 DWORD dwSize=256 ;
 HKEY hKeyChild;
 FILETIME time;
 LONG lRes;

 lRes=RegOpenKeyEx(hKeyParent,lpszKeyChild,0,KEY_ALL_ACCESS,&hKeyChild); //Open the child.
 if(lRes!=ERROR_SUCCESS)
    return lRes;
 while(RegEnumKeyEx(hKeyChild,0,szBuffer,&dwSize,NULL,NULL,NULL,&time)==S_OK) //Enumerate all of the decendents of this child.
 {
  lRes=RecursiveDeleteKey(hKeyChild,szBuffer);  //Delete the decendents of this child.
  if(lRes!=ERROR_SUCCESS)
  {
     RegCloseKey(hKeyChild);  //Cleanup before exiting.
     return lRes;
  }
  dwSize=256;
 }
 RegCloseKey(hKeyChild);      // Close the child.

 return RegDeleteKey(hKeyParent,lpszKeyChild);  //Delete this child.
}


HRESULT RegisterServer(TCHAR* szModule, const CLSID& clsid, const TCHAR* szFriendlyName, const TCHAR* szVerIndProgID, const TCHAR* szProgID)
{
 TCHAR szCLSID[CLSID_STRING_BUFFER_LENGTH];   // GetModuleFileName(hModule,szModule,sizeof(szModule)/sizeof(TCHAR))
 TCHAR szKey[64];

 #if defined MYDEBUG
 _ftprintf(fp,_T("  Entering RegisterServer()\n"));
 _ftprintf(fp,_T("    szFriendlyName = %s\n"),szFriendlyName);
 _ftprintf(fp,_T("    szVerIndProgID = %s\n"),szVerIndProgID);
 _ftprintf(fp,_T("    szProgID       = %s\n"),szProgID);
 _ftprintf(fp,_T("    szModule       = %s\n"),szModule);
 #endif
 CLSIDToTChar(clsid, szCLSID, CLSID_STRING_BUFFER_LENGTH);            //Get server location &Convert the CLSID into a char.
 _tcscpy_s(szKey, _T("CLSID\\"));                                     //Build the key CLSID\\{...}
 _tcscat_s(szKey,szCLSID);
 #if defined MYDEBUG
 _ftprintf(fp,_T("    szCLSID        = %s\n"),szCLSID);
 _ftprintf(fp,_T("    szKey   = %s\n"),szCLSID);
 #endif
 SetKeyAndValue(szKey,NULL,szFriendlyName);                           //Add the CLSID to the registry.
 SetKeyAndValue(szKey, _T("InprocServer32"), szModule);               //Add the server filename subkey under the CLSID key.
 SetKeyAndValue(szKey, _T("ProgID"), szProgID);                       //Add the ProgID subkey under the CLSID key.
 SetKeyAndValue(szKey,_T("VersionIndependentProgID"),szVerIndProgID); //Add the version-independent ProgID subkey under CLSID key.
 SetKeyAndValue(szVerIndProgID, NULL, szFriendlyName);                //Add the version-independent ProgID subkey under HKEY_CLASSES_ROOT.
 SetKeyAndValue(szVerIndProgID, _T("CLSID"), szCLSID);
 SetKeyAndValue(szVerIndProgID, _T("CurVer"), szProgID);
 SetKeyAndValue(szProgID, NULL, szFriendlyName);                      //Add the versioned ProgID subkey under HKEY_CLASSES_ROOT.
 SetKeyAndValue(szProgID, _T("CLSID"), szCLSID);
 #if defined MYDEBUG
 _ftprintf(fp,_T("  Leaving RegisterServer()\n"));
 #endif

 return S_OK ;
}


HRESULT UnregisterServer(const CLSID& clsid, const TCHAR* szVerIndProgID, const TCHAR* szProgID)
{
 TCHAR szCLSID[CLSID_STRING_BUFFER_LENGTH];
 TCHAR szKey[64];
 LONG lResult;

 #if defined MYDEBUG
 _ftprintf(fp,  _T("  Entering UnregisterServer()\n"));
 #endif
 CLSIDToTChar(clsid, szCLSID,CLSID_STRING_BUFFER_LENGTH);                //Convert the CLSID into a char.
 _tcscpy_s(szKey, _T("CLSID\\"));                                        //Build the key CLSID\\{...}
 _tcscat_s(szKey, szCLSID) ;
 lResult=RecursiveDeleteKey(HKEY_CLASSES_ROOT, szKey);                   //Delete the CLSID Key - CLSID\{...}
 lResult=RecursiveDeleteKey(HKEY_CLASSES_ROOT, szVerIndProgID);          //Delete the version-independent ProgID Key.
 lResult=RecursiveDeleteKey(HKEY_CLASSES_ROOT, szProgID) ;               //Delete the ProgID key.
 #if defined MYDEBUG
 _ftprintf(fp,  _T("  Leaving UnregisterServer()\n"));
 #endif

 return S_OK ;
}
