//Strings.h
#if !defined(STRINGS_H)
#define STRINGS_H
#define EXPANSION_FACTOR      2
#define MINIMUM_ALLOCATION   16

class String
{
 public:
 #ifndef JUST_NEED_PARSE
 friend String operator+(TCHAR*, String&);
 #endif
 String();                                //Uninitialized Constructor
 String(const TCHAR);                     //Constructor Initializes With A TCHAR.
 String(const TCHAR*);                    //Constructor Initializes String With TCHAR*
 String(const String&);                   //Constructor Initializes String With Another String (Copy Constructor)
 #ifndef JUST_NEED_PARSE
 String(const SIZE_T, bool);              //Constructor Creates String With User Specified Capacity and optionally nulls out
 String(const SIZE_T, const TCHAR);       //Constructor initializes String with int # of TCHARs
 #endif
 String(SSIZE_T);                         //Constructor initializes String with int converted to String
 String(SIZE_T);                          //Constructor initializes String with unsigned int converted to String
 #ifndef JUST_NEED_PARSE
 String(double);                          //Constructor initializes String with double converted to String
 #endif
 String& operator=(const TCHAR);          //Assign A TCHAR To A String
 String& operator=(const TCHAR*);         //Assign A Null Terminated TCHARacter Array To A String
 String& operator=(const String&);        //Assigns Another String To this One
 #ifndef JUST_NEED_PARSE
 String& operator=(int iNum);             //Assigns an int to a String
 String& operator=(unsigned int iNum);    //Assigns an unsigned int to a String
 String& operator=(double dblNum);        //Assign a double to a String
 String operator+(const TCHAR);           //For adding TCHAR to String
 String operator+(const TCHAR*);          //Adds a TCHAR* to this
 String operator+(String&);               //Adds another String to this
 String& operator+=(const TCHAR ch);      //Add TCHAR to this
 String& operator+=(const String&);       //Adds a String to this and assigns it to left of equal sign
 String& operator+=(const TCHAR*);        //Adds a TCHAR*to this and assigns it to left of equal sign
 bool operator==(String&);                //Compares Strings For Case Sensitive Equality
 #endif
 bool operator==(const TCHAR*);           //Compares String Against TCHAR* For Case Sensitive Equality
 #ifndef JUST_NEED_PARSE
 String Allocate(int);                    //Allocates a String with a specified buffer size
 String& Make(const TCHAR, int);          //Returns reference to this with iCount ch TCHARs in it
 String Left(int);                        //Returns String of iNum Left Most TTCHARs of this
 String Right(int);                       //Returns String of iNum Right Most TTCHARs of this
 String Mid(SIZE_T, SIZE_T);              //Returns String consisting of number of TTCHARs from some offset
 String Replace(TCHAR*, TCHAR*);          //Returns String with 1st TCHAR* parameter replaced with 2nd TCHAR* parameter
 String Remove(TCHAR*);                   //Returns A String With All The TCHARs In A TCHAR* Removed (Individual TCHAR removal)
 String Remove(const TCHAR*, bool);       //Returns a String with 1st parameter removed.  2nd is bool for case sensitivity.
 SIZE_T InStr(const TCHAR*, bool, bool);  //Returns one based case sensitive/insensitive offset of a particular TCHAR pStr in a String, starting from left or right (left=true)
 SIZE_T InStr(const String&, bool, bool); //Returns one based case sensitive/insensitive offset of where a particular String is in another String, starting from left or right (left=true)
 #endif
 int ParseCount(const TCHAR);             //Returns count of Strings delimited by a TTCHAR passed as a parameter
 void Parse(String*, TCHAR);              //Returns array of Strings in first parameter as delimited by 2nd TTCHAR delimiter
 #ifndef JUST_NEED_PARSE
 void SetTCHAR(unsigned, TCHAR);               //Sets TCHAR at zero based offset in this
 #endif
 void LTrim();                            //Returns String with leading spaces/tabs removed
 void RTrim();                            //Returns String with spaces/tabs removed from end
 void Trim();                             //Returns String with both leading and trailing whitespace removed
 int iVal();                              //Returns integral value of String
 SIZE_T Len();                            //Returns Length Of String Controlled By this
 SIZE_T Capacity();                       //Returns Maximum Permissable TCHARacter Count (One Less Than Allocation).
 TCHAR* lpStr();                          //Returns TCHAR* To String
 #ifndef JUST_NEED_PARSE
 void Print(bool);                        //Outputs String To Console With Or Without CrLf.
 void Print(TCHAR*, bool);                //Outputs String To Console With Or Without CrLf.
 #endif
 ~String();                               //String Destructor

 private:
 TCHAR* lpBuffer;
 SIZE_T iLen;
 SIZE_T iCapacity;
};
#ifndef JUST_NEED_PARSE
String operator+(TCHAR* lhs, String& rhs);
#endif
#endif  //#if !defined(STRINGS_H)


